<?php
/**
 * The core Common class.
 *
 * This class is used for handling all Common related functions.
 *
 *
 * @link       https://usergrowth.io/
 * @since      1.0.0
 * @package    Evergreen_Content_Poster
 * @subpackage Evergreen_Content_Poster/includes
 * @author     usergrowth <info@usergrowth.io>
 *
 *
 * $client_id = '';
 * $client_secret = '';
 * $callback_url = '';
 * $buffer = new BufferApp($client_id, $client_secret, $callback_url);
 *
 */

class Evergreen_Content_Poster_Common {

    private $option_general_settings = EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_general_settings';
    private $option_schedule_settings = EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_schedule_settings';
    private $option_timezone = EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_timezone';


    /**
     * Get general setting from wp_options
     *
     * @return array|mixed|void
     */
    public function general_settings() {

        $settings = get_option( $this->option_general_settings );

        // "false" if token does not exist (plugin is deauthorized)
        // "null" if plugin is activated first time but no connection has been made yet to Buffer
        if ( !$settings || empty( $settings ) ) {

            $settings = [
                'eg_timezone' => $this->wp_timezone(),
                'eg_position' => 'top',
                'eg_postingorder' => 'random',
                'eg_numberofdays' => 1,
                'utm_tags' => 'N'
            ];
        }

        if (!isset($settings['ecp_posting_schedule']))
            $settings['ecp_posting_schedule'] = 'daily_posts';

        return $settings;
    }

    /**
     * Save general setting to wp_options
     *
     * @param $settings
     * @return bool
     */
    public function save_settings($settings) {
        if ( ! current_user_can( 'manage_options' ) ) {
            return false;
        }

        if ( empty($settings) ) {
            return false;
        }

        return update_option( $this->option_general_settings, $settings );
    }

    /**
     * Save settings in onboard
     *
     * @param $settings
     * @return bool
     */
    public function save_settings_config($settings) {
        if ( ! current_user_can( 'manage_options' ) ) {
            return false;
        }

        if ( empty($settings) ) {
            return false;
        }

        $general_settings = get_option($this->option_general_settings);

        foreach ($settings as $key => $value) {
            $general_settings[$key] = $value;
        }

        return update_option( $this->option_general_settings, $general_settings );
    }

    /**
     * Get schedules from wp_options
     *
     * @return array|mixed|void
     */
    public function schedules() {

        $settings = get_option( $this->option_schedule_settings );

        // "false" if token does not exist (plugin is deauthorized)
        // "null" if plugin is activated first time but no connection has been made yet to Buffer
        if ( !$settings || empty( $settings ) )
            return [
                'schedules' => [],
                'generals' => []
            ];

        return $settings;

    }

    /**
     * Get Buffer plugin timezone. Else, return default WP timezone.
     */
    public function timezone() {
        $timezone = $this->load_timezone();

        // "false" if token does not exist (plugin is deauthorized)
        // "null" if plugin is activated first time but no connection has been made yet to Buffer
        if ( !$timezone || 'null' == $timezone )
            $timezone = $this->wp_timezone();

        return $timezone;
    }

    public function load_timezone() {
        return get_option( $this->option_timezone );
    }

    public function wp_timezone() {

        $current_offset = get_option( 'gmt_offset' );
        $tzstring       = get_option( 'timezone_string' );

        $check_zone_info = true;

		// Remove old Etc mappings. Fallback to gmt_offset.
        if ( false !== strpos( $tzstring, 'Etc/GMT' ) ) {
            $tzstring = '';
        }

        if ( empty( $tzstring ) ) { // Create a UTC+- zone if no timezone string exists.
            $check_zone_info = false;
            if ( 0 == $current_offset ) {
                $tzstring = 'UTC+0';
            } elseif ( $current_offset < 0 ) {
                $tzstring = 'UTC' . $current_offset;
            } else {
                $tzstring = 'UTC+' . $current_offset;
            }
        }

        return $tzstring;
    }

}