<?php
/**
 * Fired during plugin deactivation.
 *
 * This class defines all code necessary to run during the plugin's deactivation.
 *
 * @link       https://usergrowth.io/
 * @since      1.0.0
 * @package    Evergreen_Content_Poster
 * @subpackage Evergreen_Content_Poster/Deactivator
 * @author     usergrowth <info@usergrowth.io>
 */
class Evergreen_Content_Poster_Deactivator {

	/**
	 * Runs the uninstallation routines when the plugin is deactivated.
	 *
	 * @since   1.0.0
	 *
	 * @param   bool    $network_wide   Is network wide deactivation
	 */
	public static function ecp_deactivate( $network_wide ): void {
		if ( ! current_user_can( 'activate_plugins' ) )
        return;

        require_once plugin_dir_path(__FILE__) . '/class-evergreen_content_poster.php';
	    // Initialise Plugin
	    $ecp = Evergreen_Content_Poster::get_instance();

	    // Check if we are on a multisite install, activating network wide, or a single install
	    if ( ! is_multisite() || ! $network_wide ) {
	        // Single Site deactivation
	        $ecp = (new Evergreen_Content_Poster())->uninstall();
	    } else {
	        // Multisite network wide deactivation
	        $sites = get_sites( array(
	            'number' => 0,
	        ) );
	        foreach ( $sites as $site ) {
	            switch_to_blog( $site->blog_id );
	            $ecp = (new Evergreen_Content_Poster())->uninstall();
	            restore_current_blog();
	        }
	    }
	}

    /**
     * The code that unregisters the plugin settings/options, and then deletes them.
     */
    public static function unregister_settings(): void {

        // @note: the unregister_setting function always reports an error where "ecp_settings_opts" does not exist.
        // again will need to revisit this to properly use the Settings API
        // Unregister these settings with the plugin
        // unregister_setting( EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_settings_opts', EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_options');

        // remove plugin options
        delete_option( 'ecp_options' );
        delete_option( 'ecp_buffer_token' );
        delete_option( 'ecp_buffer_profiles' );
        delete_option( 'ecp_general_settings' );
        delete_option( 'ecp_schedule_settings' );
        delete_option( 'ecp_timezone' );
        delete_option( 'ecp_buffer_pre_profile' );
        delete_option( 'ecp_buffer_lifetime' );
        delete_option( 'ecp_frequency_schedules' );
    }
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-evergreen_content_poster-deactivator.php
 */
function deactivate_evergreen_content_poster($network_wide): void {
    // Call the cron schedule
    $sche = new Evergreen_Content_Poster_Cron_Scheduler();
    $sche->unschedule_cron_event();

     // remove plugin setting options
    delete_option(EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_date_last_active');
    delete_option(EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_date_last_timestamp');

    $token = evergreenGetStatisticsToken(false);

    $data['sites'] = array(
        'token' => $token,
        'url' => home_url(),
        'deactivation_date' => date('Y-m-d H:i:s'),
        'status' => 'deactivated',
        'version' => EVERGREEN_CONTENT_POSTER_PLUGIN_VERSION,
        'plugin_name' => 'evergreen content poster'
    );

    updateApiStatistics($data);
}

/**
 * The code that runs during plugin uninstall
 * Plugin deactivate and uninstall does the same things for now.
 * Will be updated in the future to accomodate extra changes for uninstall.
 */
function uninstall_evergreen_content_poster($network_wide): void {

    Evergreen_Content_Poster_Deactivator::ecp_deactivate($network_wide);
    // remove plugin setting options
    Evergreen_Content_Poster_Deactivator::unregister_settings();
}
