<?php
/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @link       https://usergrowth.io/
 * @since      1.0.0
 * @package    Evergreen_Content_poster
 * @subpackage Evergreen_Content_Poster/includes
 * @author     usergrowth <info@usergrowth.io>
 */
class Evergreen_Content_Poster
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Evergreen_Content_Poster_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

    private $plugin_admin;

    private $buffer_service;
    private $common_service;

    private $option_general_settings = EVERGREEN_CONTENT_POSTER_KEY_PREFIX . '_general_settings';

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		if ( defined( 'EVERGREEN_CONTENT_POSTER_PLUGIN_VERSION' ) ) {
			$this->version = EVERGREEN_CONTENT_POSTER_PLUGIN_VERSION;
		} else {
			$this->version = '1.0.0';
		}

		$this->plugin_name = EVERGREEN_CONTENT_POSTER_PLUGIN_NAME;

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Evergreen_Content_Poster_Loader. Orchestrates the hooks of the plugin.
	 * - Evergreen_Content_Poster_i18n. Defines internationalization functionality.
	 * - Evergreen_Content_Poster_Admin. Defines all hooks for the admin area.
	 * - Evergreen_Content_Poster_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies(): void {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-evergreen_content_poster-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-evergreen_content_poster-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-evergreen_content_poster-admin.php';

		/**
		 * The wrapper class responsible for defining all methods for buffer integration.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-evergreen_content_buffer.php';

		$this->loader = new Evergreen_Content_Poster_Loader();

        $this->buffer_service = new Evergreen_Content_Poster_Buffer();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Evergreen_Content_Poster_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale(): void {

		$plugin_i18n = new Evergreen_Content_Poster_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}
	
	/**
	 * HTML
	 *
	 * The non-conformant markup this object represents. Most of the sections and
	 * options can be created using one of the existing ecp_{Item} classes.
	 * Sometimes we need something that does not fit those boxes. This class
	 * provides native methods for a few of those cases, and an ecp_add_HTML() method
	 * for everything else.
	 *
	 * @var string $html
	 *
	 */
	public $html = '';
	
	/**
	 * Allows custom HTML to be added.
	 *
	 * @since  1.1.3
	 * @param  string $html Required: The fully qualified, ready-to-print HTML to display.
	 * @return object $this This object for method chaining.
	 *
	 */
	public function ecp_add_HTML( $html ) {
		if ( !is_string( $html) ) :
			$this->_throw( 'This requires a string of HTML!' );
		endif;

		$this->html .= $html;

		return $this;
	}	
	
	/**
	 * A method for creating the admin sidebar HTML.
	 *
	 * @since  1.1.3
	 * @param  void
	 * @return string The compiled HTML returned as a string.
	 *
	 */
	public function ecp_do_admin_sidebar() {
		$ecp_status_title = __( 'Press Ctrl+C to Copy this information.', 'evergreen-content-poster' );

		//* This is an array of fully qualified HTML strings, ready to print.

		$html = '<div class="ecp-system-status-wrapper">';
			$html .= '<h4><?= $ecp_status_title ?></h4>';
			$html .= '<div class="ecp-system-status-container"> '. $this->ecp_system_status() . '</div>';
		$html .= '</div>';
		$html .= '<div class="ecp-system-status--logfile-wrapper">';
			$html .= '<h4><?= $ecp_status_title ?></h4>';
			$html .= '<div class="ecp-system-status--logfile-container" style="overflow: scroll"> '. $this->ecp_status_logfile() . '</div>';
		$html .= '</div>';
		
		return $this->html = $html;
	}
		
	/**
	 * A method for compiling the system status html.
	 *
	 * @since  1.1.3
	 * @param  void
	 * @return string The compiled html returned as a string.
	 *
	 */
	private function ecp_system_status() {

		/**
		 * System Status Generator
		 */
		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugins = get_plugins();
		$pluginList = '';

		foreach ( $plugins as $plugin ) :
			$pluginList .= '<tr><td><b><a href="' . $plugin['PluginURI'] . '" target="_blank">' . $plugin['Name'] . '</a></b></td><td>' . $plugin['Version'] . '</td></tr>';
		endforeach;

		if ( function_exists( 'fsockopen' ) ) :
			$fsockopen = '<span style="color:green;">Enabled</span>';
		else :
			$fsockopen = '<span style="color:red;">Disabled</span>';
		endif;

		if ( function_exists( 'curl_version' ) ) :
			$curl_version = curl_version();
			$curl_status = '<span style="color:green;">Enabled: v' . $curl_version['version'] . '</span>';
		else :
			$curl_status = '<span style="color:red;">Disabled</span>';
		endif;

        if($this->buffer_service->get_token()) {
            $profiles = $this->buffer_service->profiles;
            $profiles = json_decode($profiles);
            if (!empty($profiles)) :
                $profile_lists = '';
                foreach ($profiles as $service => $profile_list) :
                    foreach ($profile_list as $key => $single_profile) :
                        $service_type = ucwords($single_profile->service) . ' ' . ucwords($single_profile->service_type);
                        $profile_lists .= $service_type . ': ' . $single_profile->formatted_username . '<br /> ';
                    endforeach;
                endforeach;
                $profile_lists = trim($profile_lists, ', ');
            endif;


            $buffer_status = '<span style="color:green;">Connected</span>';
            $buffer_token = '<span style="color:green;">' . $this->buffer_service->get_token() . '</span>';
            $buffer_profiles = '<span style="color:green;">' . $profile_lists . '</span>';
        } else {
            $buffer_status = '<span style="color:red;">No Buffer Connection</span>';
            $buffer_token = '<span style="color:red;">No access token</span>';
            $buffer_profiles = '<span style="color:red;">No profiles connected</span>';
        }


		$theme = wp_get_theme();

		$system_status = '
			<table style="width:100%;">
				<tr><td colspan="2"><h2>Environment Statuses</h2></td></tr>
				<tr><td><b>Home URL</b></td><td>' . get_home_url() . '</td></tr>
				<tr><td><b>Site URL</b></td><td>' . get_site_url() . '</td></tr>
				<tr><td><b>WordPress Version</b></td><td>' . get_bloginfo( 'version' ) . '</td></tr>
				<tr><td><b>PHP Version</b></td><td>' . phpversion() . '</td></tr>
				<tr><td><b>WP Memory Limit</b></td><td>' . WP_MEMORY_LIMIT . '</td></tr>
				<tr><td><b>Evergreen Content Poster Version</b></td><td>' . EVERGREEN_CONTENT_POSTER_PLUGIN_VERSION . '</td></tr>
				<tr><td colspan="2"><h2>Connection Statuses</h2></td></tr>
				<tr><td><b>fsockopen</b></td><td>' . $fsockopen . '</td></tr>
				<tr><td><b>cURL</b></td><td>' . $curl_status . '</td></tr>
				<tr><td><b>Buffer</b></td><td>' . $buffer_status . '</td></tr>
				<tr><td><b>Buffer token available</b></td><td>' . $buffer_token . '</td></tr>
				<tr><td><b>Buffer profiles</b></td><td>' . $buffer_profiles . '</td></tr>
				<tr><td colspan="2"><h2>Theme information</h2></td></tr>
				<tr><td><b>Theme Name</b></td><td>' . $theme['Name'] . '</td></tr>
				<tr><td><b>Theme Version</b></td><td>' . $theme['Version'] . '</td></tr>
				<tr><td colspan="2"><h2>Active Plugins</h2></td><td></td></tr>
				<tr><td><b>Number of Active Plugins</b></td><td>' . count( $plugins ) . '</td></tr>
				<tr><td colspan="2"><h2>Active plugin names and version number</h2></td></tr>
				' . $pluginList . '
			</table>
			';

		return $system_status;
	}
	
	/**
	 * A method for compiling the status logfile overview html.
	 *
	 * @since  1.3.0
	 * @param  void
	 * @return string The compiled html returned as a string.
	 *
	 */
	private function ecp_status_logfile() {
	
	global $wpdb;
	$logmessages = $wpdb->get_results("SELECT profile_name, result, result_message, status_text, status_created_at, request_sent FROM {$wpdb->prefix}evergreen_content_logs");

	$text = '';
	foreach ($logmessages as $log) {
	    $text .= '<tr><td>' . $log->profile_name . '</td><td>' . $log->result . '</td><td>' . $log->result_message . '</td><td>' . $log->status_text . '</td><td>' . $log->status_created_at . '</td></tr>';
    }

	$status_logfile = '<table style="width:100%;" border="1">
			<tr><td colspan="5"><h2>Sharing logs</h2></td></tr>
			<tr><td><b>Profile</b></td><td><b>Result</b></td><td><b>Result Message</b></td>
			<td><b>Social Status Text</b></td><td><b>Date</b></td></tr>' . $text . '</table>';
	
	return $status_logfile;
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks(): void {

		$plugin_admin = new Evergreen_Content_Poster_Admin( $this->get_plugin_name(), $this->get_version() );

        $this->plugin_admin = $plugin_admin;

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'add_evergreen_metabox' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'evergreen_content_poster_save_fields' );

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'evergreen_content_poster_admin_menu' );

        // for receiving response from api.evergreencontentposter.io
        $this->loader->add_action( 'wp_ajax_nopriv_save_ajax_token', $plugin_admin, 'save_ajax_token' );
        $this->loader->add_action( 'wp_ajax_save_ajax_token', $plugin_admin, 'save_ajax_token' );

        // for adding network posts for WP posts
        $this->loader->add_action('wp_ajax_nopriv_create_network_post', $plugin_admin, 'create_network_post');
        $this->loader->add_action( 'wp_ajax_create_network_post', $plugin_admin, 'create_network_post' );

        $this->loader->add_action('wp_ajax_nopriv_create_network_post_share', $plugin_admin, 'create_network_post_share');
        $this->loader->add_action( 'wp_ajax_create_network_post_share', $plugin_admin, 'create_network_post_share' );

        $this->loader->add_action('wp_ajax_nopriv_delete_network_post_share', $plugin_admin, 'delete_network_post_share');
        $this->loader->add_action( 'wp_ajax_delete_network_post_share', $plugin_admin, 'delete_network_post_share' );
        // add_action('wp_ajax_ajaxInsertData', [$this, 'ajaxInsertData']);

        $this->loader->add_action('wp_ajax_nopriv_is_share_status_save', $plugin_admin, 'is_share_status_save');
        $this->loader->add_action( 'wp_ajax_is_share_status_save', $plugin_admin, 'is_share_status_save' );

        $this->loader->add_action('wp_ajax_nopriv_profile_status_save', $plugin_admin, 'profile_status_save');
        $this->loader->add_action( 'wp_ajax_profile_status_save', $plugin_admin, 'profile_status_save' );

        // for deleting network posts for WP posts
        $this->loader->add_action('wp_ajax_nopriv_delete_network_post', $plugin_admin, 'delete_network_post');
        $this->loader->add_action('wp_ajax_delete_network_post', $plugin_admin, 'delete_network_post');

        // for updating the settings
        $this->loader->add_action( 'wp_ajax_nopriv_save_settings', $plugin_admin, 'save_settings' );
        $this->loader->add_action( 'wp_ajax_save_settings', $plugin_admin, 'save_settings' );

        $this->loader->add_action( 'wp_ajax_nopriv_save_settings_config', $plugin_admin, 'save_settings_config' );
        $this->loader->add_action( 'wp_ajax_save_settings_config', $plugin_admin, 'save_settings_config' );

        // Share immediately
        $this->loader->add_action( 'save_post', $plugin_admin, 'ecp_share_post_upon_publication' );

        $this->loader->add_action( 'admin_head', $plugin_admin, 'not_connected_social_notification' );

        $this->loader->add_action( 'admin_notices', $plugin_admin, 'ecp_wp_cron_notice' );
    }

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
     * @access   public
	 */
	public function run(): void {
        
        $this->loader->run();

        // if buffer access token exists as GET param from API
        if ( isset($_REQUEST['ba_response']) && !empty($_REQUEST['ba_response']) ) {
            
            $token = sanitize_text_field($_REQUEST['ba_response']);

            $this->buffer_service->save_token($token);

            wp_redirect('/wp-admin/admin.php?page=evergreen-content-settings#AccountSettings');
            wp_redirect('/wp-admin/admin.php?page=' . $_REQUEST['page']);
//            wp_redirect('/wp-admin/admin.php?page=evergreen-content-configurator');
        }

        // remove the stored Buffer authorization token
        if ( isset($_REQUEST['action']) && 'ba_remove' == strtolower( $_REQUEST['action'] ) ) {
            $this->buffer_service->deauthorize();

            wp_redirect('/wp-admin/admin.php?page=' . $_REQUEST['page'] . '&removed_auth=Y');
        }
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since    1.0.0
     * @access   public
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since    1.0.0
     * @access   public
	 * @return    Evergreen_Content_Poster_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since    1.0.0
     * @access   public
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

    /**
     * Holds the class object.
     *
     * @since    1.0.0
     * @access   public
     * @var     object
     */

    public static $instance;

    /**
     * Returns the singleton instance of the class.
     *
     * @since    1.0.0
     * @access   public
     * @return  object Class.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
        	self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Update plugin specific options
     *
     * @since    1.0.0
     * @access   public
     */
	public function evergreen_content_poster_update_option( $option_name, $new_value ): void {
		if ( get_option( $option_name ) !== false ) {
			// The option already exists, so we just update it.
			update_option( $option_name, $new_value );
		} else {
			// The option hasn't been added yet. We'll add it with $autoload set to 'no'.
			add_option( $option_name, $new_value, $deprecated = null, $autoload = 'no' );
		}
	}

    // @todo: move plugin install feature to separate class
    /**
     * Runs installation routines for first time users
     *
     * @since    1.0.0
     * @access   public
     */
	public function install(): void {

		$this->add_evergreen_content_poster_defaults();
	}

	/**
	 * Check Evergreen Content Poster installation and set options
     *
     * @since    1.0.0
     * @access   public
	 */
	public function add_evergreen_content_poster_defaults(): void
	{
		global $wpdb;

		$options = get_option('ecp_options');

        if ( !isset($options['ecp_installed']) || $options['ecp_installed'] != 1 || !is_array($options) )
        {
            $opt = array(
                'ecp_version' => EVERGREEN_CONTENT_POSTER_PLUGIN_VERSION,
                'ecp_db_version' => EVERGREEN_CONTENT_POSTER_DB_VERSION,
                'premium' => 0
            );
            $opt = array_merge($opt, $options);

            if ( is_multisite() )
            {
                update_site_option( 'ecp_options', $opt );
            } else {
                $this->evergreen_content_poster_update_option( 'ecp_options', $opt );
            }
        }

        $this->evergreen_content_poster_db_install();
	}

    /**
     * Sets the wpdb tables to the current blog
     *
     * @since    1.0.0
     * @access   public
     */
    public function evergreen_content_poster_set_wpdb_tables(): void
    {
        global $wpdb;

        $wpdb->evergreen_content_logs = $wpdb->prefix . 'evergreen_content_logs';
        $wpdb->evergreen_categories = $wpdb->prefix . 'evergreen_categories';
    }

	/**
	 * Creates or updates the Evergreen Content Poster DB tables
     *
     * @since    1.0.0
     * @access   public
	 */
	public function evergreen_content_poster_db_install(): void
	{
		global $wpdb;

		$charset_collate = $wpdb->get_charset_collate();

		$this->evergreen_content_poster_set_wpdb_tables();

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->evergreen_content_logs} (
		  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
		  `post_id` int(11) NOT NULL,
		  `action` enum('publish','update','repost','bulk_publish', 'share', 'add_queue') DEFAULT NULL,
		  `request_sent` datetime NOT NULL,
		  `profile_id` varchar(191) NOT NULL,
		  `profile_name` varchar(191) NOT NULL DEFAULT '',
		  `result` enum('success','test','pending','warning','error') NOT NULL DEFAULT 'success',
		  `result_message` text,
		  `status_text` text,
		  `status_image` text,
		  `status_link` text,
		  `status_created_at` datetime DEFAULT NULL,
		  `status_due_at` datetime DEFAULT NULL,
		  `ecp_posting_schedule` text,
		  PRIMARY KEY (`id`),
		  KEY `post_id` (`post_id`),
		  KEY `action` (`action`),
		  KEY `result` (`result`),
		  KEY `profile_id` (`profile_id`)
		) {$charset_collate};";

		dbDelta($sql);

        $sqlCate = "CREATE TABLE IF NOT EXISTS {$wpdb->evergreen_categories} (
              `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `name` varchar(191) NOT NULL DEFAULT '',
              `description` text,
              `random_availability` int(1) NOT NULL DEFAULT 1,
              `status` int(1) NOT NULL DEFAULT 1,
              `is_default` int(1) DEFAULT NULL,
              `created_at` datetime DEFAULT NULL,
              `updated_at` datetime DEFAULT NULL,
              PRIMARY KEY (`id`)
            ) {$charset_collate};";

        dbDelta($sqlCate);

        // Check categories
        $categories = $wpdb->get_results("SELECT * FROM {$wpdb->evergreen_categories}");
        if(empty((array) $categories)) {
            // add default Category
            $dateNow = date('Y-m-d H:i:s');
            $insertSql = "INSERT INTO {$wpdb->evergreen_categories} (name, description, random_availability, status, is_default, created_at, updated_at) VALUES ('Your evergreen content', 'Use this category to promote your own content - share any blog posts or pages you publish.', 1, 1, 1, '${dateNow}', null), ('Promotional', 'Use this category to promote your services/products. Make sure to include Calls To Actions (CTAs) like \"Book a call with us\" or \"start a trial account\".', 1, 1, 0, '${dateNow}', null), ('Quotes, Questions, and Fun', 'Use this category to generate engagement on your socials. Share inspirational or fun quotes, from your niche or create open questions for your audience.', 1, 1, 0, '${dateNow}', null)";
            dbDelta($insertSql);
        }
	}

    // @todo: move plugin uninstall feature to separate class
    /**
     * Runs uninstallation routines
     *
     * @since    1.0.0
     * @access   public
     */
    public function uninstall(): void {

        $this->uninstall_evergreen_content_poster();

        // $this->evergreen_content_poster_unset_wpdb_tables();
    }

	/**
	 * Unsets the wpdb tables to the current blog
	 *
     * @since    1.0.0
     * @access   public
	 */
	public function evergreen_content_poster_unset_wpdb_tables(): void
	{
	    global $wpdb;

	    unset($wpdb->evergreen_content_logs);
	    unset($wpdb->evergreen_categories);
	}

	/**
	 * Drops or uninstalls the Evergreen Content Poster tables
     *
     * @since    1.0.0
     * @access   public
	 */
	public function uninstall_evergreen_content_poster(): void
	{
		if ( ! current_user_can( 'activate_plugins' ) )
	        return;

        // global $wp_meta_boxes;

        $this->evergreen_content_poster_set_wpdb_tables();

        // dbDelta does not support DROP TABLE
        global $wpdb;

        $sqlDeleteLogs = "DROP TABLE IF EXISTS `{$wpdb->evergreen_content_logs}`;";
        $wpdb->query($sqlDeleteLogs);
        
        $sqlDeleteCategories = "DROP TABLE IF EXISTS `{$wpdb->evergreen_categories}`;";
        $wpdb->query($sqlDeleteCategories);

        $ecp_options = get_option('ecp_options');

        // update statistic api
        $token = evergreenGetStatisticsToken();
        $data['sites'] = array(
            'url' => home_url(),
            'token' => $token,
            'deinstall_date' => date('Y-m-d H:i:s'),
            'status' => 'deleted',
            'version' => EVERGREEN_CONTENT_POSTER_PLUGIN_VERSION,
//            'contact_email' => isset($ecp_options['contact_email']) ? $ecp_options['contact_email'] : '',
            'plugin_name' => 'evergreen content poster'
        );

        updateApiStatistics($data);

        // @todo: add command to delete option value for plugin DB version
        // delete_option("my_plugin_db_version");

        // $this->evergreen_content_poster_unset_wpdb_tables();
	}

}
